package org.average.whereami;

import org.average.whereami.CredentialStore;

import java.lang.Math;

import com.google.api.client.extensions.android2.AndroidHttp;
import com.google.api.client.auth.oauth2.draft10.AccessTokenResponse;
import com.google.api.client.googleapis.auth.oauth2.draft10.GoogleAccessProtectedResource;
import com.google.api.client.http.HttpResponseException;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.http.javanet.NetHttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.jackson.JacksonFactory;
import com.google.api.services.latitude.Latitude;
import com.google.api.services.latitude.LatitudeRequest;
import com.google.api.services.latitude.model.Location;

import android.content.Context;
import android.content.SharedPreferences;
import android.content.res.Resources;
import android.text.format.Time;
import android.util.Log;

public final class LastLocation extends Oracle {

	final String TAG = getClass().getName();

	private Latitude latitude;

	private String[] loc_names;
	private String[] loc_lats;
	private String[] loc_lons;
	private String[] loc_dists;
	private String myname;
	private String timestamp;
	private String longago;
	private String lessthanhour;
	private String onehour;
	private String hours2to4;
	private String hours5up;

	public LastLocation(final Resources res, final CredentialStore store) {
		loc_names = res.getStringArray(R.array.loc_names);
		loc_lats = res.getStringArray(R.array.loc_lats);
		loc_lons = res.getStringArray(R.array.loc_lons);
		loc_dists = res.getStringArray(R.array.loc_dists);
		myname = res.getString(R.string.myname);
		timestamp = res.getString(R.string.timestamp);
		longago = res.getString(R.string.longago);
		lessthanhour = res.getString(R.string.lessthanhour);
		onehour = res.getString(R.string.onehour);
		hours2to4 = res.getString(R.string.hours2to4);
		hours5up = res.getString(R.string.hours5up);

		HttpTransport transport = AndroidHttp.newCompatibleTransport();
		JsonFactory jsonFactory = new JacksonFactory();
		AccessTokenResponse token = store.read();
		Log.v(TAG, "tokens - access: \"" + token.accessToken +
			"\", refresh: \"" + token.refreshToken +
			"\", client_id: \"" + ClientCredentials.CLIENT_ID +
			"\", client_secret: \"" +
					ClientCredentials.CLIENT_SECRET +
			"\"");
		GoogleAccessProtectedResource accessProtectedResource =
			new GoogleAccessProtectedResource(
				token.accessToken,
				transport, jsonFactory,
				ClientCredentials.CLIENT_ID,
				ClientCredentials.CLIENT_SECRET,
				token.refreshToken)
				{
					@Override
					public void onAccessToken(
							String accessToken) {
						Log.v(TAG, "Update access token to \"" + accessToken + "\"");
						store.updateAccessToken(
							accessToken);
					}
				}
				;
		Latitude.Builder lbldr = Latitude.builder(transport,
								jsonFactory);
		lbldr.setHttpRequestInitializer(accessProtectedResource);
		lbldr.setApplicationName("WhereAmI/1.0");
		latitude = lbldr.build();
	}

	@Override
	public final String getResult() {
		try {
			Log.v(TAG, "entering getResult");
			Latitude.CurrentLocation.Get request =
				latitude.currentLocation().get();
			request.setGranularity("best");
			Location cloc = request.execute();
			Double lat = Double.parseDouble(
					cloc.getLatitude().toString());
			Double lon = Double.parseDouble(
					cloc.getLongitude().toString());
			Long tsm = Long.parseLong(
					cloc.getTimestampMs().toString());
			return locationMessage(lat, lon, tsm);
                } catch (Exception ex) {
			Log.v(TAG, "exception in getResult: " + ex);
			ex.printStackTrace();
			return ex.getMessage();
                }
	}

	private String locationMessage(Double lat, Double lon, Long tsm) {
		Log.v(TAG, "entering locationMessage(" + lat +
				"," + lon + "," + tsm + ")");
		String locmsg = "<UNSET>";
		String agomsg = "<UNSET>";
		for (int i = 0; i < loc_names.length; i++) {
			Double dist = haversine(lat, lon,
					Double.parseDouble(loc_lats[i]),
					Double.parseDouble(loc_lons[i]));
			Log.v(TAG, "Dist from " + loc_names[i] +
				" is " + dist);
			if ((loc_dists[i] == "0") ||
			    (dist < Double.parseDouble(loc_dists[i]))) {
				locmsg = loc_names[i];
				break;
			}
		}
		Long ago = (System.currentTimeMillis() - tsm) / 3600000;
		if (ago < 1) {
			agomsg = lessthanhour;
		} else if (ago < 2) {
			agomsg = onehour;
		} else if (ago < 5) {
			agomsg = "" + ago + " " + hours2to4;
		} else if (ago < 24) {
			agomsg = "" + ago + " " + hours5up;
		} else {
			agomsg = longago;
		}
		return myname + " " + locmsg + ", " + timestamp +
			" " + agomsg;
	}

	private Double toRad(Double deg) {
		return deg * 3.14159265 / 180;
	}

	private Double haversine(Double lat1, Double lon1,
				Double lat2, Double lon2) {
		Double dlat = toRad(lat2 - lat1);
		Double dlon = toRad(lon2 - lon1);
		Double a = Math.sin(dlat / 2) * Math.sin(dlat / 2) +
			Math.sin(dlon / 2) * Math.sin(dlon / 2) *
			Math.cos(toRad(lat1)) * Math.cos(toRad(lat2));
		Double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
		return 6371 * c;
	}
}
